import unittest

class TestParserContextInNewStackFrames(unittest.TestCase):
	@classmethod
	def setUpClass(cls):
		cls._harena_t = gdb.lookup_type("HArena")
		cls._arena = gdb.parse_and_eval("malloc(sizeof(HArena))").cast(cls._harena_t.pointer()).dereference() # Quick and dirty way to allocate something in the process GDB is debugging (the inferior)
		cls._hps_t = gdb.lookup_type("HParseState")
		cls._test_parse_state = gdb.parse_and_eval("malloc(sizeof(HParseState))").cast(cls._hps_t.pointer()).dereference()
		# TODO: make this independent of endianness and pointer size
		gdb.selected_inferior().write_memory(int(cls._test_parse_state.address)+(cls._hps_t['arena'].bitpos//8), int(cls._arena.address).to_bytes(8, 'little')) # horrible, horrible way to set the 
		cls._hps_t = cls._test_parse_state.address.dereference() # "refresh" the gdb.Value, since the effects of .write_memory() don't seem to show up

	def setUp(self):
		self.a = Parser("a", 128)
		self.b = Parser("b", 32)
		# TODO: could self.harena_t and self.arena be initialized in setUpClass?
		self.harena_t = gdb.lookup_type("HArena")
		#self.arena = gdb.Value(bytearray(self.harena_t.sizeof), self.harena_t) # Initialize a new HArena to all zeros
		self.arena = gdb.parse_and_eval("malloc(sizeof(HArena))").cast(self.harena_t.pointer()).dereference() # Quick and dirty way to allocate something in the process GDB is debugging (the inferior)
		self.top_level_parse = TopLevelParse()
		self.top_level_parse.parser_objs[32] = self.b
		self.top_level_parse.parser_objs[128] = self.a
		# TODO: the above two lines simulate a side effect of enter_h_do_parse. could be avoided if self.a and self.b was assigned after the calls below

		self.top_level_parse.enter_h_packrat_parse(self.a.address)

		# TODO: When using DETAILED_ARENA_STATS, arena needs to be an actual HArena object, not just "256"
		#TODO: are we initializing from scratch for every test?
		self.top_level_parse.enter_h_do_parse(400, self.arena.address, self.a.address)
		self.top_level_parse.parse_virtual(self.a.address)
		self.top_level_parse.enter_perform_lowlevel_parse(self.a.address)

	#TODO: teardown, invoke the return_from_* functions

	def test_top_of_stack(self):
		#self.assertEqual(self.top_level_parse.peek_parser(), cr)
		pass

	def test_after_h_do_parse(self):
		self.top_level_parse.enter_h_do_parse(400, self.arena.address, self.b.address)
		self.assertIs(self.top_level_parse.peek_parser(), self.b)

	def test_after_enter_parse_virtual(self):
		self.top_level_parse.enter_h_do_parse(400, self.arena.address, self.b.address)
		self.top_level_parse.parse_virtual(self.b.address)
		self.assertIs(self.top_level_parse.peek_parser(), self.b)

	def test_after_enter_perform_lowlevel_parse(self):
		self.top_level_parse.enter_h_do_parse(400, self.arena.address, self.b.address)
		self.top_level_parse.parse_virtual(self.b.address)
		self.top_level_parse.enter_perform_lowlevel_parse(self.b.address)
		self.assertIs(self.top_level_parse.peek_parser(), self.b)

class TestParserContextWhenReturningFromStackFrames(unittest.TestCase):
	def setUp(self):
		self.a = Parser("a", 128)
		self.b = Parser("b", 32)
		self.harena_t = gdb.lookup_type("HArena")
		self.arena = gdb.parse_and_eval("malloc(sizeof(HArena))").cast(self.harena_t.pointer()).dereference() # Quick and dirty way to allocate something in the process GDB is debugging (the inferior)
		self.hps_t = gdb.lookup_type("HParseState")
		self.test_parse_state = gdb.parse_and_eval("malloc(sizeof(HParseState))").cast(self.hps_t.pointer()).dereference()
		# TODO: make this independent of endianness and pointer size
		gdb.selected_inferior().write_memory(int(self.test_parse_state.address)+(self.hps_t['arena'].bitpos//8), int(self.arena.address).to_bytes(8, 'little')) # horrible, horrible way to set the 
		self.hps_t = self.test_parse_state.address.dereference() # "refresh" the gdb.Value, since the effects of .write_memory() don't seem to show up
		self.top_level_parse = TopLevelParse()
		self.top_level_parse.parser_objs[32] = self.b
		self.top_level_parse.parser_objs[128] = self.a

		# TODO: the above two lines simulate a side effect of enter_h_do_parse. could be avoided if self.a and self.b was assigned after the calls below

		self.top_level_parse.enter_h_packrat_parse(self.a.address)

		#TODO: are we initializing from scratch for every test?
		self.top_level_parse.enter_h_do_parse(self.test_parse_state.address, self.arena.address, self.a.address)
		self.top_level_parse.parse_virtual(self.a.address)
		self.top_level_parse.enter_perform_lowlevel_parse(self.a.address)
		self.top_level_parse.enter_h_do_parse(self.test_parse_state.address, self.arena.address, self.b.address)
		self.top_level_parse.parse_virtual(self.b.address)
		self.top_level_parse.enter_perform_lowlevel_parse(self.b.address)

	def test_after_return_from_perform_lowlevel_parse(self):
		self.top_level_parse.return_from_perform_lowlevel_parse()
		self.assertIs(self.top_level_parse.peek_parser(), self.b)

	def test_after_return_from_h_do_parse(self):
		hps_t = gdb.lookup_type("HParseState")
		#test_parse_state = gdb.Value(b'\x00'*80, hps_t)
		test_parse_state = gdb.parse_and_eval("malloc(sizeof(HParseState))").cast(hps_t.pointer()).dereference()

		self.top_level_parse.return_from_perform_lowlevel_parse()
		self.top_level_parse.return_from_h_do_parse(self.test_parse_state.address, self.b.address, 0)
		self.assertIs(self.top_level_parse.peek_parser(), self.a)
